local myName, AddOn = ...
local version = GetAddOnMetadata(myName, "Version")

local StatPriority = {}
StatPriority.__index = StatPriority

function StatPriority:new(title)
    local self = {};
    setmetatable(self, StatPriority);
    self.Title = title
    self.Categories = {}
    self.HasDefaultOnly = true
    return self
end

function StatPriority:SetDefaultPriority(priority)
    return self:SetPriority("", priority)
end

function StatPriority:SetPriority(category, priority)
    if category ~= "" then
        self.HasDefaultOnly = false
    end
    self.Categories[category] = priority
    return self
end

function StatPriority:GetPriorityString(category)
    if self.Categories[category] then
        local s = self.Categories[category]
        s = gsub(s, "Strength", "STR")
        s = gsub(s, "Agility", "AGI")
        s = gsub(s, "Intelligence", "INT")
        s = gsub(s, "Stamina", "STA")
        return s
    end
end

AddOn.IsRetail = WOW_PROJECT_ID == WOW_PROJECT_MAINLINE
AddOn.IsClassic = WOW_PROJECT_ID == WOW_PROJECT_CLASSIC
AddOn.IsBCC = WOW_PROJECT_ID == WOW_PROJECT_BURNING_CRUSADE_CLASSIC
AddOn.IsWotLK = WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC
AddOn.StatTable = {}

function AddOn:AddStatPriority(key, title)
    self.StatTable[key] = StatPriority:new(title)
    return self.StatTable[key]
end

function AddOn:GetStatPriority()
    local _, key = UnitClass("player")
    if self.IsRetail then
        local specId = GetSpecializationInfo(GetSpecialization()) or 0
        key = key .. "-" .. specId
    end
    return self.StatTable[key]
end

function AddOn:ShowStatPriority(tooltip, class, priority)
    if not priority then
        tooltip:Hide()
        return
    end

    tooltip:SetOwner(tooltip:GetParent(), "ANCHOR_TOPRIGHT")
    tooltip:ClearLines()

    local classColor = RAID_CLASS_COLORS[class]
    tooltip:AddLine(
        priority.Title or class,
        classColor.r, classColor.g, classColor.b)

    if priority.HasDefaultOnly then
        tooltip:AddLine(
            priority:GetPriorityString(""),
            HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
    else
        for category in pairs(priority.Categories) do
            tooltip:AddDoubleLine(
                category == "" and "PVE" or category,
                priority:GetPriorityString(category),
                NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
                HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
        end
    end

    tooltip:Show()
end

local f = CreateFrame("GameTooltip", myName, PaperDollFrame, "GameTooltipTemplate")

function f:OnEvent(event, ...)
    self[event](self, ...)
end

function f:ADDON_LOADED(addOnName)
    if addOnName == myName then
        PaperDollFrame:HookScript("OnShow", function() self:Update() end)
    end
end

function f:SPELLS_CHANGED()
    self:Update()
end

function f:Update()
    local priority = AddOn:GetStatPriority()
    local _, class = UnitClass("player")
    AddOn:ShowStatPriority(self, class, priority)
end

f:RegisterEvent("SPELLS_CHANGED")
f:RegisterEvent("ADDON_LOADED")
f:SetScript("OnEvent", f.OnEvent)

print(myName, version, "Loaded!")